#!/usr/bin/env python
# -*- coding:utf-8 -*-
import json
import logging
from datetime import datetime

import requests

from braincube.bc_connector.config_handler import get_domain, get_verify

logger = logging.getLogger(__name__)


class Requester:
    def __init__(self, braincube_name, sso_token):
        self.braincube_name = braincube_name
        self.sso_token = sso_token
        self.headers = {'Content-Type': 'application/json', 'Accept': "application/json", 'IPLSSOTOKEN': sso_token}

    def request_data(self, ref, selected_variables, start_date, end_date):
        # Return the result of the request of data retrieving, construct with the params of the user
        if ref['referenceDate'] not in selected_variables:
            selected_variables.append(ref['referenceDate'])
        url = "{base_url}/{mb_id}/LF".format(base_url=self.get_api_url("braindata"),
                                             mb_id=ref['name'])
        filters = [ref['order'], start_date, end_date]
        content = {
            "order": ref['referenceDate'],
            "definitions": selected_variables,
            "context":
                {
                    "dataSource": ref['name'],
                    "filter":
                        {
                            "BETWEEN": filters
                        }
                }
        }
        self.log_request("data", url, str(content))
        start_time = datetime.now().timestamp()
        result = requests.post(url, data=json.dumps(content), headers=self.headers, verify=get_verify())
        end_time = datetime.now().timestamp()
        logger.debug("[request_data][duration: time to first byte: %s s, time to last byte: %s s]",
                     "{0:.2f}".format(result.elapsed.microseconds / 1e6),
                     "{0:.2f}".format(end_time - start_time))
        return json.loads(result.text)

    def get_references(self, mb_id):
        # Return the reference of the memorybase selected
        url = "{base_url}/mb{mb_id}/simple".format(base_url=self.get_api_url("braindata"),
                                                   mb_id=str(mb_id))
        self.log_request("reference", url)
        res = requests.get(url, headers=self.headers, verify=get_verify())
        return json.loads(res.text)

    def get_data_defs(self, mb_id):
        # Return the definitions of the variables for the selected memorybase
        url = "{base_url}/mb/{mb_id}/variables/selector".format(base_url=self.get_api_url("braincube"),
                                                                mb_id=str(mb_id))
        self.log_request("datadefs", url)
        result = requests.get(url, headers=self.headers, verify=get_verify())
        return json.loads(result.text)['items']

    def get_memorybase(self):
        # Return the memorybases available for the selected braincube
        url = "{base_url}/mb/all/selector".format(base_url=self.get_api_url("braincube"))
        self.log_request("memorybase", url)
        result = requests.get(url, headers=self.headers, verify=get_verify())
        return json.loads(result.text)['items']

    def log_request(self, type, url, content="None"):
        logger.debug("Request {type}: [url: {url}, headers: {headers}, content: {content}]".format(type=type,
                                                                                                  url=url,
                                                                                                  headers=self.headers,
                                                                                                  content=content))

    def get_api_url(self, destination):
        return "https://api.{domain}/braincube/{braincube_name}/{destination}".format(domain=get_domain(),
                                                                                      braincube_name=self.braincube_name,
                                                                                      destination=destination)



import json
import logging
import os
from os.path import join

from braincube.bc_connector.certificate_gen import gen_ssl_cert_if_none

logger = logging.getLogger(__name__)

config_dir = os.path.expanduser('~/.braincube')
file_path = join(config_dir, "config.json")
config_template = {'client_secret': '', 'client_id': '', 'domain': 'mybraincube.com', 'verify': True}

config = None


def config_setup_ok():
    """
    Generate a self-signed ssl certificate if none are found in ~./braincube
    Read the local config file and check if it's filled
    If no file is found one is created
    If the file is not filled a console message is displayed
    """
    if os.path.isfile(file_path):
        gen_ssl_cert_if_none(config_dir)
        read_local_config()
        return is_config_filled(config)

    else:
        logger.warn("No configuration file found. ")
        init_config_file()


def init_config_file():
    logger.info("Configuration file created in: " + file_path)
    os.makedirs(os.path.dirname(file_path), exist_ok=True)
    with open(file_path, "w") as outfile:
        json.dump(config_template, outfile)
    logger.warn("Current domain: '" + config_template["domain"] + "' change it to add your sub-domain if you have one")
    logger.warn("Please fill the client_id and the client_secret and try again")


def is_config_filled(config):
    if config is None or not config['client_id'] or not config['client_secret']:
        logger.info("client_id and/or client_secret are empty, please fill them in: " + file_path)
        return False
    else:
        logger.info("config filled with: %s", config)
        return True


def read_local_config():
    logger.info("Reading configuration from: " + file_path)
    with open(file_path) as json_data_file:
        global config
        config = json.load(json_data_file)


def _read_config_from_dict(conf):
    global config
    config = conf


def get_client_id():
    return config['client_id']


def get_client_secret():
    return config['client_secret']


def get_verify():
    global config
    verify = config.get('verify', True)
    if isinstance(verify, bool):
        return verify
    elif isinstance(verify, str):
        if len(verify) > 0:
            return verify
        else:
            logger.warn('Invalid verify value: "{}", keep checking certficates'.format(verify)) 
    return True


def get_domain():
    if config is None:
        return ""
    else:
        return config['domain']




# class TestConfigHandler(TestCase):
from braincube.bc_connector.config_handler import is_config_filled, get_verify, _read_config_from_dict



def test_is_config_filled_ko_01():
    assert is_config_filled({"client_secret": "", "client_id": ""}) is False


def test_is_config_filled_ko_02():
    assert is_config_filled({"client_secret": "", "client_id": "test"}) is False


def test_is_config_filled_ko_03():
    assert is_config_filled({"client_secret": "test", "client_id": ""}) is False


def test_is_config_filled_ok():
    assert is_config_filled({"client_secret": "test", "client_id": "test"}) is True


def test_no_verify():
    global config

    config_verify = {"client_secret": "test", "client_id": "test", 'verify':True}
    config_no_verify = {"client_secret": "test", "client_id": "test", 'verify':False}
    config_certchain = {"client_secret": "test", "client_id": "test", 'verify':'hi mom'}
    config_none_certchain = {"client_secret": "test", "client_id": "test", 'verify':None}
    config_emptycertchain = {"client_secret": "test", "client_id": "test", 'verify':''}

    _read_config_from_dict(config_verify)
    assert get_verify() is True
    
    _read_config_from_dict(config_no_verify)
    assert get_verify() is False

    _read_config_from_dict(config_certchain)
    assert get_verify() == 'hi mom'

    # cases where the config is bad
    _read_config_from_dict(config_none_certchain)
    assert get_verify() is True

    _read_config_from_dict(config_emptycertchain)
    assert get_verify() is True




from braincube.bc_connector.data_formatting import data_to_dataframe, get_data_with_id

fakeRef = {'referenceDate': 'mb888/d99999992',
           'timezone': 'Etc/GMT',
           'hfUrl': 'http://TESTING:9999/MX_FAKE',
           'datadefs': [{'type': 'NUMERIC', 'id': 'mb888/d99999991'},
                        {'type': 'DATETIME', 'id': 'mb888/d99999992'},
                        {'type': 'DATETIME', 'id': 'mb888/d99999993'},
                        {'type': 'DISCRET', 'id': 'mb888/d99999994'},
                        {'type': 'NUMERIC', 'id': 'mb888/d99999995'},
                        {'type': 'DISCRET', 'id': 'mb888/d99999996'},
                        {'type': 'DISCRET', 'id': 'mb888/d99999997'},
                        {'type': 'NUMERIC', 'id': 'mb888/d99999998'},
                        {'type': 'NUMERIC', 'id': 'mb888/d99999999'}],
           'reference': 'mb888/d99999991',
           'order': 'mb888/d99999992',
           'name': 'mb888',
           'lfUrl': 'http://TESTING:9999/MX_FAKE'}

fakeData = [{'type': 'DATETIME',
             'id': 'mb888/d99999992',
             'data': ['20120102_000000', '20120103_000000', '20120103_000000']},
            {'type': 'NUMERIC',
             'id': 'mb888/d99999998',
             'data': ['377.4', '376.4', '376.4']},
            {'type': 'DISCRET',
             'id': 'mb888/d99999994',
             'data': ['IMMÉDIAT', 'NORMAL', 'URGENT']}
            ]

fakeDataDef = [{'bcId': 99999999, 'local': 'Age du Ticlet en mois', 'tag': 'Age du Ticlet en mois', 'id': 99999999,
                'standard': 'Age du Ticlet en mois'},
               {'bcId': 99999998, 'local': 'Age du ticket', 'tag': 'Age du ticket', 'id': 99999998,
                'standard': 'Age du ticket'},
               {'bcId': 99999997, 'local': 'Tracker', 'tag': 'Tracker', 'id': 99999997, 'standard': 'Tracker'},
               {'bcId': 99999992, 'local': 'created', 'tag': 'created', 'id': 99999992, 'standard': 'created'},
               {'bcId': 99999995, 'local': 'duration', 'tag': 'duration', 'id': 99999995, 'standard': 'duration'},
               {'bcId': 99999991, 'local': 'N° de Ticket', 'tag': 'issue', 'id': 99999991, 'standard': 'N° de Ticket'},
               {'bcId': 99999994, 'local': 'priority', 'tag': 'priority', 'id': 99999994, 'standard': 'priority'},
               {'bcId': 99999993, 'local': 'solved', 'tag': 'solved', 'id': 99999993, 'standard': 'solved'},
               {'bcId': 99999996, 'local': 'status', 'tag': 'status', 'id': 99999996, 'standard': 'status'}]


def test_map_data():
    tt = get_data_with_id(fakeData)
    assert tt.get('99999992')[2] == "20120103_000000"
    assert tt.get('99999994')[2] == "URGENT"
    assert tt.get('99999998')[2] == "376.4"


def test_data_formating():
    test = data_to_dataframe(fakeData, fakeDataDef, fakeRef)
    assert test.columns[0] == "Age du ticket"
    assert test.columns[1] == "priority"
    assert test.values[0][0] == "377.4"
    assert test.values[0][1] == "IMMÉDIAT"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import logging
from datetime import datetime

import pandas as pd

logger = logging.getLogger(__name__)


def get_data_with_id(data):
    # Transforms the data into a dictionary with the variable's id and the corresponding data for that variable
    var = {}
    for i in data:
        var[i['id'].split('/d')[1]] = i['data']
    return var


def data_to_dataframe(datas, datadefs, ref):
    logger.debug('Start converting to Pandas dataFrame')
    start_time = datetime.now().timestamp()
    # Return a dataframe based on the given data
    data = get_data_with_id(datas)
    index = []
    result = {}
    for datadef in datadefs:
        intent = str(datadef['id'])
        if intent in data.keys():
            extent = data[intent]
            if intent == ref['referenceDate'].split('/d')[1]:
                for j in range(len(extent)):  # Changes the format of the date to be readable
                    extent[j] = datetime.strptime(extent[j], "%Y%m%d_%H%M%S") \
                        .strftime("%Y/%m/%d_%H:%M:%S")
                index = extent
            else:
                result[datadef['local']] = extent
    dataframe = pd.DataFrame(result, index=index)

    end_time = datetime.now().timestamp()
    duration = "{0:.2f}".format(end_time - start_time)
    logger.info("Conversion to Pandas DataFrame in " + str(duration) + " s")
    return dataframe
